/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/EnvironmentalEffects/DisturbanceHistoryMessage/BasicImpl/CallbackInfo.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::CallbackInfo.
 *
 * This file contains the declaration of and documentation for the
 * RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::CallbackInfo class.
 */

#ifndef RTTC_EnvironmentalEffects_DisturbanceHistoryMessage_BasicImpl_CallbackInfo_h
#define RTTC_EnvironmentalEffects_DisturbanceHistoryMessage_BasicImpl_CallbackInfo_h

#include <DCT/System/config.h>
#include <basicImpl_RTTC_EnvironmentalEffects_v1_EXPORT.h>
#include <list>
#include <ostream>
#include <RTTC/EnvironmentalEffects/DisturbanceHistoryMessage/Interface.h>
#include <DCT/Utils/SmartPtr.h>

namespace RTTC
{
  namespace EnvironmentalEffects
  {
    namespace DisturbanceHistoryMessage
    {
      namespace BasicImpl
      {
        /*!\class CallbackInfo CallbackInfo.h \
           RTTC/EnvironmentalEffects/DisturbanceHistoryMessage/BasicImpl/CallbackInfo.h
         *
         * \brief CallbackInfo object passed to all callbacks
         *
         * This is a convenience class to simplify access to the callbacks.
         * Whatever state is encapsulated here is accessible to all the
         * callbacks.  A smart pointer is passed to all the factories,
         * which in turn pass it to created callbacks.
         *
         * The basic implementation owns a std::list of message pointers.
         * The MessageCallback can be modified to stick received message
         * pointers onto this list.  This implementation is also initialized
         * with reference to an OutputStream, which can be used to represent
         * a view object that can be updated when a message is received.
         *
         * A user application may modify the CallbackInfo in any way desired,
         * and the encapsulated information will be available to all callbacks.
         */
        class basicImpl_RTTC_EnvironmentalEffects_v1_EXPORT CallbackInfo
        {
        public:
          /*!
            \param[in] os A std::ostream to which output will be written.
            \param[in] verbosity Controls the amount of output generated.
            Default is zero.
          */
          CallbackInfo(
            std::ostream & os,
            unsigned int verbosity = 0 );

          //! Instances live for at least the duration of a subscription.
          ~CallbackInfo();

          //! Return reference to the outputstream.
          std::ostream &
          getOutputStream();

          //! Return a reference to the list maintained in the CallbackInfo.
          std::list< RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::Pointer > &
          getReceivedMessageList();

          //! Return the verbosity for output
          /*!
           * 0 - No output
           * 1 - Callback evoked
           * 2 - 1 plus publication state
           */
          unsigned int
          getVerbosity();

        private:
          //! Default copy construction is disallowed.
          CallbackInfo( CallbackInfo const & rhs );
          //! Default copy assignment is disallowed.
          CallbackInfo & operator=( CallbackInfo const & rhs );

          //! Hold a reference to an ostream
          std::ostream & _outputStream;

          //! Hold a list of the received Message pointers
          std::list< RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::Pointer > _receivedMessageList;

          //! Verbosity for output
          unsigned int _verbosity;
        };

        /*! \brief A reference-counted "smart" pointer to a
         * CallbackInfo.
         *
         * TENA uses a variety of smart pointers to simplify the management
         * of memory associated with objects returned by the middleware. This
         * type is constructed from a raw call to new() so that user code
         * cannot easily access a deleted object or leak an object.
         */
        typedef DCT::Utils::SmartPtr< CallbackInfo > CallbackInfoPtr;

      } // End of namespace BasicImpl
    } // End of namespace DisturbanceHistoryMessage
  } // End of namespace EnvironmentalEffects
} // End of namespace RTTC

/****************************************************************************/
/*** Inline definition of REALLY simple functions                   *********/
/****************************************************************************/

///////////////////////////////////////////////////////////////////////////////
inline
std::list< RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::Pointer > &
RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::
CallbackInfo::
getReceivedMessageList()
{
  return _receivedMessageList;
}

///////////////////////////////////////////////////////////////////////////////
inline
std::ostream &
RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::
CallbackInfo::
getOutputStream()
{
  return _outputStream;
}

///////////////////////////////////////////////////////////////////////////////
inline
unsigned int
RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::
CallbackInfo::
getVerbosity()
{
  return _verbosity;
}

#endif // ifndef RTTC_EnvironmentalEffects_DisturbanceHistoryMessage_BasicImpl_CallbackInfo_h
